public extension Mic {
  /// Следует доставить состояния звуковой активности элементам
  ///
  /// Условия:
  /// 1. Изменились даты активности
  /// 2. Элемент запросил свою дату активности при наличии валидной даты
  /// 2. Элемент запросил свою дату активности без наличия валидной даты
  ///
  /// - Returns: Словарь состояний
  static func shouldDeliverActivityDates(_ c: MicContext) -> [String: Date]? {
    if let ad = shouldResetActivityDates(c) {
      return ad
    }

    if
      let id = c.requestActivityDate,
      let timeout = c.activityDates.value[id],
      timeout > Date()
    {
      return [id: timeout]
    }

    if let id = c.requestActivityDate {
        return [id: .distantPast]
    }

    return nil
  }

  /// Следует обновить словарь состояний звуковой активности
  ///
  /// Условия:
  /// 1. Пришли id активных участников
  ///
  /// - Returns: Словарь активных состояний
  static func shouldResetActivityDates(_ c: MicContext) -> [String: Date]? {
    if c.activeIds.isRecent {
      var ad = c.activityDates.value
      let ids = c.activeIds.value
      let now = Date()
      // Задаём абсолютную дату истечения для активных id.
      for id in ids {
        ad[id] = now + C.activityTimeout
      }

      // Собираем истёкшие id.
      var expiredIds = [String]()
      for (id, timeout) in ad {
        if timeout < now {
          expiredIds.append(id)
        }
      }
      // Удаляем истёкшие id.
      for id in expiredIds {
        ad[id] = nil
      }

      return ad
    }

    return nil
  }
}
